#include "defdefs.h"

.global baud_rate_div100
.equ baud_rate_div100,(BAUD_RATE/100)

.global clock
.equ clock,CLOCK

;------------------------------------------------------------------------
.section reset_00

#ifdef DEBUGGER
#ifdef BANNER
    pbegin
    pprint BANNER
    pend
#endif
#endif

#ifdef ENABLE_WATCHDOG
    ldi r16,0x0F    ; enable watchdog timer (~2 sec.)
    out wdtcr,r16
    wdr             ; reset watchdog timer
#endif

;------------------------------------------------------------------------
.section reset_end

#ifdef DEBUGGER
#ifndef NO_ETHER_ADDR_ON_BOOT
    rcall show_ether
    rcall do_crlf
#endif
#endif
;------------------------------------------------------------------------
.section fast

#ifdef DEBUGGER
    clr r0
    sts putc_b,r0
    rcall command_check
#endif

#ifdef ENABLE_WATCHDOG
    wdr             ; reset watchdog timer
#endif
;------------------------------------------------------------------------
.section slow

#if defined(USE_BOOTP)
    rcall bootp_timer
#endif

;------------------------------------------------------------------------
.text
;+
; **-pwdr-pcode watchdog reset.
;
; inputs:
;       none
;-
    .global pwdr
    .global swdr
pwdr:   pcode_routine   0
swdr:
#ifdef ENABLE_WATCHDOG
    wdr             ; reset watchdog timer
#endif
    ret
;------------------------------------------------------------------------
.text
#ifndef DEBUGGER
    .global tcpget_cmdline
tcpget_cmdline:
    pret
#endif
;------------------------------------------------------------------------
.section eeprom
#ifndef BREADBOARD
    .global eeprom_ether
eeprom_ether:
#include "ether.inc"
#endif

    .global baud_rate_divisor
baud_rate_divisor:
    .byte SERIAL_BAUD_DIVISOR

;------------------------------------------------------------------------
.text
.global unknown_tcp_port
unknown_tcp_port:
#ifdef UNKNOWN_TCP_PORT_HANDLER
    pjump UNKNOWN_TCP_PORT_HANDLER
#else
    pret
#endif

.global poksetip
#ifndef POKSETIP
poksetip: pcode_routine 0
    clr r16        ; set Z-flag (i.e., SETIP changes allowed)
    ret
#else
.equ poksetip,POKSETIP
#endif

;------------------------------------------------------------------------
#ifdef SERIAL_FULL_PARITY

#ifndef SER_PARITY
#define SER_PARITY 2
#endif

#ifndef SER_STOP
#define SER_STOP 1
#endif

#ifndef SER_PAR_ERR
#define SER_PARITY 0
#endif

.section eeprom
.global baud_rate
baud_rate:   .word baud_rate_div100  ; baud rate / 100
.global parity
parity:      .byte SER_PARITY        ; 0=even, 1=odd, 2=none
.global stop_bits
stop_bits:   .byte SER_STOP          ; 1 or 2
.global parity_err
parity_err:  .byte SER_PAR_ERR       ; 0=OK, 1=drop, 2=set MSB

#endif /* SERIAL_FULL_PARITY */
;------------------------------------------------------------------------
