;--------------------------------------------------------------------------
;+
; **-pprinturl-print parameter string in URL 
;
; pprinturl poffset,mode
; inputs:
;   poffset   = address of word in sram to get offset in receive buffer
;   mode      = print mode as follows:
;                 0: normal mode
;                 1: map \r and \n to CR and LF respectively (\\ -> \)
;
; outputs:
;   poffset = updated offset in receive buffer (just after string)
;
; notes:
;   (1) character in URL output until '&' or ' ' encountered
;   (2) %XX changed to character defined by hex digits "XX"
;-
;--------------------------------------------------------------------------

#define T1 (BP+0)
#define T2 (T1+2)
#define T3 (T2+2)
#define TEMPS_END (T3+2)
#define N_TEMPS (TEMPS_END-T1)

#define P1  TEMPS_END
#define P2  (P1+2)
#define P3  (P2+2)

#define poffset   P1
#define SD_MODE   P2
#define SD_LAST   P2+1
#define SD_LEN    P3
#define SD_CHAR   T1
#define SD_TEMP   T2
#define SD_OFF    T3

#ifdef DEBUG
#define D
#else
#define D ;
#endif

.text
    .global pprinturl
pprinturl: pcode_routine 2

    rcall pip_setup_r10reloc
    pjump pnt_url

.section seeprom
pnt_url:
    penter N_TEMPS
    pmovw SD_OFF,[poffset]

    D pserial_debug

    pmovbi SD_LAST,0                    ; clear last character
pnturl_next:
    pwdr                                ; reset hardware watchdog timer
    D pprintv "SD_OFF=",[SD_OFF]
    purl2s SD_CHAR,[SD_OFF],1           ; get next byte
    D pprintv "SD_CHAR=",[SD_CHAR]
    pjumpne pnturl_exit                 ; exit if none
    D pprint " pprinturl match\r\n"

    pcmpbi SD_CHAR,' '                  ; at the end?
    pjumpeq pnturl_exit                 ; SPACE...exit
    pjumplo pnturl_exit                 ; <SPACE...exit
    pcmpbi SD_CHAR,'&'                  ; the end? '&'?
    pjumpeq pnturl_exit                 ; yes...exit

    pcmpbi SD_CHAR,'+'                  ; map '+' -> ' '
    pjumpne not_plus
    pmovbi SD_CHAR,' '
    pjump output_char_now
not_plus:

    pcmpbi SD_CHAR,'%'                  ; check HTTP quoting char (%XX)
    pjumpne output_next_char
    pincw SD_OFF                        ; skip '%'
    purl2s SD_CHAR,[SD_OFF],2           ; get XX of %XX
    pincw SD_OFF                        ; adjust offset

    pmovb SD_TEMP,SD_CHAR               ; get first char
    pandwi SD_TEMP,0xff
    psubwi SD_TEMP,'0'                  ; convert to binary (maybe)
    pcmpwi SD_TEMP,10                   ; see if valid hex digit
    pjumplo do_upper_digit              ; yep - do upper one
    pandwi SD_TEMP,0xf                  ; 1-6
    paddwi SD_TEMP,9                    ; turn 1-6 into 10-15
do_upper_digit:
    pmovb SD_CHAR,SD_CHAR+1             ; get second digit
    pandwi SD_CHAR,0xff                 ; isolate it
    psubwi SD_CHAR,'0'                  ; convert to binary (maybe)
    pcmpwi SD_CHAR,10                   ; see if valid hex digit
    pjumplo merge_digits                ; yep - merge the two
    pandwi SD_CHAR,0xf
    paddwi SD_CHAR,9
merge_digits:
    pshnw SD_TEMP,4                     ; shift top bits into place
    paddwi SD_CHAR,[SD_TEMP]            ; SD_CHAR now has char

output_next_char:
    pandwi SD_CHAR,0xff                 ; isolate it

    pcmpbi SD_MODE,0                    ; special \ processing?
    pjumpeq output_char_now             ; mode == 0...no special processing

    pcmpbi SD_LAST,0x5c                 ; last character a back-slash?
    pjumpne check_for_backslash         ; no...proceed as normal

    pcmpwi SD_CHAR,'r'                  ; got a \r?
    pjumpne not_CR                      ; no...skip ahead
    pmovbi SD_CHAR,0x0d                 ; yes...change it to CR
not_CR:
    pcmpwi SD_CHAR,'n'                  ; got \n?
    pjumpne not_LF                      ; no...
    pmovbi SD_CHAR,0x0a                 ; yes...change to LF
not_LF:
    pcmpwi SD_CHAR,0x5c                 ; got BACKSLASH?
    pjumpeq output_char_now             ; yes...output it now

check_for_backslash:
    pcmpwi SD_CHAR,0x5c                 ; back-slash?
    pjumpeq skip_output                 ; yes...don''t output

output_char_now:
    pputcb SD_CHAR                      ; output character
skip_output:
    pmovbi SD_LAST,[SD_CHAR]            ; save "last character"
    pincw SD_OFF                        ; adjust pointer

    pjump pnturl_next                   ; back for more

pnturl_exit:
    D pprint " pprinturl exit\r\n"
    pmovwi [poffset],[SD_OFF]           ; update offset
    pretn N_TEMPS

