
;-------------------------------------------------------------------------
; Copyright (c) 1999 Steven Freyder, David Helland, Bruce D. Lightner and
; Lightner Engineering.  Use subject to licensing restrictions described
; the file LICENSE.txt.  If you do not agree with the licensing terms then
; you must return and/or destroy this software immediately!
;-------------------------------------------------------------------------

#ifdef SERIAL_ALWAYS_BINARY
#undef SERIAL_FULL_PARITY
#undef SERIAL_DEBUGGER
#endif

#ifdef SERIAL_FULL_PARITY
#define SET_PARITY_ON_FRAMING_ERRORS
#endif

#define BUF_WRAP  (max_nch-1)

.dseg

.global nch
.global inbuf
bss nch,1
bss inbuf,max_nch

#if !defined(SERIAL_ALWAYS_BINARY)
.global serial_mode
bss serial_mode,1   ; serial port mode 0=command, non-zero=binary
.global ready
bss ready,1
#endif
;
; com_in and com_out MUST be adjacent!
;
.global com_in
bss com_in,1
.global com_out
bss com_out,1           ; com buffer in/out indices
.global com_lost
bss com_lost,1           ; count of lost bytes
.global xoff
bss xoff,1           ; zero if not xoffed
#if defined(SERIAL_DEBUGGER)
.global ccaddr
bss ccaddr,2           ; address where Control-C pressed
#endif /* SERIAL_DEBUGGER */

#ifdef SERIAL_FULL_PARITY
.global serflags
bss r_parity,1         ;  0=even, 1=odd, 2=none
bss r_parity_err,1     ;  0=OK, 1=drop, 2=set MSB
#endif /* SERIAL_FULL_PARITY */

.cseg

;+
; **-serial_init-serial port initialization for reset.
;-

    .section reset_io
    rcall serial_init
    .text

.global pserial_init
.global serial_init
pserial_init: pcode_routine 0
serial_init:

#ifdef SERIAL_FULL_PARITY
; grab flags from EEPROM for later parity processing
    movwi   y,parity
    rcall   rdeeb
    sts     r_parity,i0
    movwi   y,parity_err
    rcall   rdeeb
    sts     r_parity_err,i0
#endif /* SERIAL_FULL_PARITY */

; set serial port baud rate.

    movwi   y,baud_rate_divisor
    rcall   rdeeb
    out     UBRR,i0

; enable transmitter and receiver, and enable receiver interrupts.

#if !defined(SERIAL_ALWAYS_BINARY)
    clr r10l
    rjmp pser_mode_init
#else
    ldi r17,(1<<rxcie)|(1<<rxen)|(1<<txen)     ; enable receive interrupts
    out UCR,r17
    ret
#endif

com_circular:
    lds r17,com_in      ; get IN index
    lds r18,com_out     ;  and OUT index
    inc r17
    andi r17,BUF_WRAP   ; wrap it
    cp r17,r18          ; check it
    breq count_lost     ; sorry - we lost a char

#ifndef SET_PARITY_ON_FRAMING_ERRORS
#ifndef IGNORE_FRAMING_ERRORS
    in r18,usr          ; get UART status register
    bst r18,fe          ; check for framing error (skip if cleared)
    brts ignore_char    ; FE=1...ignore character
#endif
#endif

    pushw z
    clr zh
    lds zl,com_in
    addwi z,inbuf
#ifdef SET_PARITY_ON_FRAMING_ERRORS
    in r18,usr          ; get UART status register
    bst r18,fe          ; check for framing error (skip if cleared)
    in r18,udr          ; read character from UART
    brtc 1f             ; FE=0...keep char
#if 0
    ldi r18,'?'         ; FE=1...map to "?"
#else
    ori r18,0x80           ; FE=1...set sign bit
#endif
1:
#else /* !SET_PARITY_ON_FRAMING_ERRORS */
    in r18,udr
#endif /* !SET_PARITY_ON_FRAMING_ERRORS */
#if defined(ISR_ECHO)
    out udr,r18
#endif
    st z,r18
    popw z

    sts com_in,r17      ; advance IN index
    rjmp com_bail       ; bail out
count_lost:
    lds r17,com_lost
    inc r17
    sts com_lost,r17
ignore_char:
    in r18,udr          ; dump the byte to clear interrupt
    rjmp com_bail

.global VEC_rx_isr
VEC_rx_isr:
rx_isr:
    push w
    push    r17
    push    r18
    in      r18,sreg
    push r18
#if defined(SERIAL_ALWAYS_BINARY)
    rjmp com_circular
#endif

#if defined(SERIAL_DEBUGGER)
    lds r18,serial_mode
    tst r18
    brne com_circular
    in      r17,udr
    cpi     r17,3       ; Control-C does a reset
    breq    go_reset
    push    yl
    push    yh
    cpi r17,0x13    ; control-S - set xof flag
    breq go_xoff
    cpi r17,0x11    ; control-Q - clear xof flag
    breq go_xoff
    lds     r18,ready
    tst     r18
    brne    rxignore
    cpi     r17,0x0d
    brne    nocr
is_cr:
    sts     ready,r17
    rjmp    rxignore
go_xoff:
    andi    r17,2       ; isolate on/off bit
    sts xoff,r17
    rjmp rxignore
nocr:
    cpi r17,0x0a        ; also allow LF
    breq is_cr
    lds     yl,nch
    ldi yh,0
    cpi     yl,max_nch
    brlo    rxok        ; unsigned compare
    ldi     r17,'!'
    rjmp    rxecho
rxok:
    mov     r18,yl
    addwi y,inbuf
    st      y,r17
    inc     r18
    sts     nch,r18
rxecho:
#ifndef SERIAL_ECHO_OF
    out     udr,r17
#endif
rxignore:
    pop     yh
    pop     yl
#else /* !SERIAL_DEBUGGER */
    rjmp com_circular
#endif /* !SERIAL_DEBUGGER */

com_bail:
    pop     r18
    out     sreg,r18
    pop     r18
    pop     r17
    pop w
    reti
#if defined(SERIAL_DEBUGGER)
go_reset:
    pop r17
    pop r17
    pop r17
    pop r17

; stack is in same state as on initial entry to interrupt.

    pop yh
    pop yl
    stsw ccaddr,y
    clr r0
    out     UCR,r0          ; disable everything
    rjmp control_c
#endif /* SERIAL_DEBUGGER */
;+
; **-pser_mode-flush buffers and set mode on serial port.
;
; inputs:
;   r10l = mode as: 0-normal, non-zero-enable pass-all (binary) mode.
;
; outputs:
;   buffer flushed and characters being stored appropriately.
;-

#if !defined(SERIAL_ALWAYS_BINARY)
    .global pser_mode
pser_mode:  pcode_routine   1
pser_mode_init:
    ldi r17,(1<<rxen)|(1<<txen)                ; disable receive interrupts
    out UCR,r17

    clr r10h
    sts com_in,r10h	                       ; reset buffer pointers
    sts com_out,r10h
    sts nch,r10h                               ; reset command buffer

    tst r10l                                   ; set serial_mode using r10l
    breq psb_normal
    inc r10h
psb_normal:
    sts serial_mode,r10h

    ldi r17,(1<<rxcie)|(1<<rxen)|(1<<txen)     ; enable receive interrupts
    out UCR,r17
    ret
;+
; **-pser_putc-pcode serial port putchar.
;
; inputs:
;   r10 = character to transmit
;
; outputs:
;   transmission to serial port initiated.
;
; notes:
;   waits for a "transmit done" indication on the serial port
;   before transmitting this character.
;-

    .global pser_putc
pser_putc:  pcode_routine 1
    mov ch,r10l
    push r17
    rjmp do_put1
;+
; **-pser_getc-pcode serial port getchar.
;
; outputs:
;   r10 -> resulting byte
;   condition codes set so that a jumpeq will take if buffer empty.
;-
    .global ser_getc
    .global pser_getc
pser_getc:  pcode_routine   1
ser_getc:
    lds r17,com_in
    lds r18,com_out
    cp r17,r18
    brne pser_getc_got_byte
    rjmp set_pcode_flags
pser_getc_got_byte:
    rcall r10reloc          ; r10->x where to store it
    mov zl,r18              ; get OUT index low 1/2
    clr zh                  ; clear top 1/2
    addwi z,inbuf           ; bias for buffer
    ld r12l,z               ; get the byte
#ifdef SERIAL_FULL_PARITY
    lds r17,r_parity        ; check parity (even/odd or none?)
                            ;   0=even, 1=odd, 2=none
    bst r17,1               ; move bit 1 to T-flag
    brts 1f                 ; jump if bit 1 set (no parity)
    ldi r17,0x7f            ; bit 1 clear...zap parity bit
    and r12l,r17
1:
#endif /* SERIAL_FULL_PARITY */
    st x,r12l               ; store byte
    inc r18
    andi r18,BUF_WRAP       ; wrap it
    sts com_out,r18         ; store it back
    clz
    rjmp set_pcode_flags    ; jumpne will take now
#endif /* !SERIAL_ALWAYS_BINARY */
;+
; **-serial_putchar-serial port putchar.
;
; inputs:
;   ch  = character (r25)
#ifdef KEEP_PUTC_OK
;   putcok  = 0 if putchar is disabled
#endif
;   xoff    = 0 if not xofed
;
; outputs:
;   character transmission started if appropriate
;
; notes:
;   this routine is called from the base PICOWEB module.
;-

.global import_serial_putchar
.global serial_putchar

import_serial_putchar:
serial_putchar:
    push r17
#ifdef KEEP_PUTC_OK
    lds r17,putcok
    tst r17
    breq putc_off
#endif
wait_xon:
    lds r17,xoff            ; check xof flag
    or r17,r17
    brne wait_xon
do_put1:
#ifdef SERIAL_FULL_PARITY
    push r10
    push r11
    push r16
    lds r16,r_parity        ; check parity (even/odd or none?)
    sbrc r16,1              ; skip if bit 1 clear
    rjmp 1f                 ; bit 2 set...parity none
    andi ch,0x7f            ; clear parity bit
    rcall calc_par          ; calculate parity
    sbrc r16,0              ; skip if flag bit 1 clear (even)
    ori ch,0x80             ; odd...set sign bit
    eor ch,r10              ; XOR in parity bit
1:  pop r16
    pop r11
    pop r10
#endif /* SERIAL_FULL_PARITY */
2:  in      r17,usr
    bst     r17,udre        ; check if data register empty
    brtc    2b
    out     udr,ch          ; start transmission
    sbi     usr,txc         ; clear "TX complete" status bit in UART status reg
#ifdef KEEP_PUTC_OK
putc_off:
#endif
    pop r17
    ret

#ifdef SERIAL_FULL_PARITY
;+
; **-calc_parity-calculate parity of a character
;
; inputs:
;   ch  = character (r25)
;
; outputs:
;   r10 = parity bit (0x10 or 0x00)
;   r11 & r17 destroyed
;-
calc_par:
    push ch        ; save ch
    clr r11        ; r11 = 0 (count of ones)
    clr r10        ; r10 = 0
    ldi r17,8      ; loop 8 times
    clc            ; C=0
1:  ror ch         ; rotate ch thru C
    adc r11,r10    ; count += 0 + C
    dec r17        ; decrement count
    brne 1b        ; back for more?
    ror ch         ; restore original ch
    ror r11        ; C <= LSB(count)
    ror r10        ; MSB(r10) <= C (i.e., 0x80 or 0x00)
    pop ch         ; restore ch
    ret
#endif /* SERIAL_FULL_PARITY */
