;-------------------------------------------------------------------------
; Copyright (c) 1999 Steven Freyder, David Helland, Bruce D. Lightner and
; Lightner Engineering.  Use subject to licensing restrictions described
; the file LICENSE.txt.  If you do not agree with the licensing terms then
; you must return and/or destroy this software immediately!
;-------------------------------------------------------------------------

#include "tcpip.inc"
#include "defdefs.h"

.set buf_len,48

.data

global_common buf,buf_len

.text

;+
; **-tcp_recv-user TCP receive handler.
;
; inputs:
;   packet in receive buffer
;   putchar output redirected to network
;
; outputs:
;   packet written to transmit buffer using pputc/equivalent
;   must return with RET.
;-

    .global import_tcp_recv
    .global tcp_recv
import_tcp_recv:
tcp_recv:
    rcall pcode
    pcall tcp_recv_1
    pasmret

;
; seeprom_unlocked  meaning
; ----------------  -------
;      0x00         locked (no net access to eeprom/seeprom)
;      0x01         partly unlocked (net write of eeprom OK)
;      0xff         completely unlocked (net read/write eeprom/seeprom OK)
;

tcp_recv_1:
    pee2s buf+2,seeprom_unlocked,1 ; get state of "unlock" byte
    pr2s buf,TCP_DP,2            ; get destination port

    pcmpwi buf,BSWAP(TCPPORT_CMD); check for debugger command line port
    pjumpeq tcpget_cmdline

    pcmpbi buf+2,0               ; check lock state
    pjumpeq tcp_skip_load_dump   ; unlock=0...no network load/dump

    pcmpwi buf,BSWAP(612)        ; check for EEPROM loader port
    pjumpeq seeprom_loader

    pcmpbi buf+2,0x01            ; check lock state
    pjumpeq tcp_skip_load_dump   ; unlock=1...all locked except EEPROM write

    pcmpwi buf,BSWAP(611)        ; check for SEEPROM loader port
    pjumpeq seeprom_loader

    pcmpwi buf,BSWAP(613)        ; check for SEEPROM dumper port
    pjumpeq seeprom_dumper

    pcmpwi buf,BSWAP(614)        ; check for EEPROM dumper port
    pjumpeq seeprom_dumper

    pcmpbi buf+2,0xff            ; check lock state
    pjumpeq unknown_tcp_port     ; unlock=0xff...no HTTP access!

tcp_skip_load_dump:
#if !defined(NO_HTTP)
    pcmpwi buf,BSWAP(80)         ; check for www/http
    pjumpne unknown_tcp_port     ; unknown port (use UNKNOWN_TCP_PORT_HANDLER)
 ;
 ; build the URL content in the TCP_DATA area of the transmit buffer.
 ;
    pmovwi buf,[data_addr]
    paddwi buf,4                 ; skip over "GET "
    pr2s buf,[buf],buf_len       ; save first n bytes of request
#if defined(SHOW_HTTP_GET)
    pmovbi putc_b,0
    pprinta 0,[data_addr],30     ; show first 30 of get request
    pmovbi putc_b,1
#endif
    pjump process_url
#else /* !NO_HTTP */
    pjump unknown_tcp_port       ; unknown port (use UNKNOWN_TCP_PORT_HANDLER)
#endif /* !NO_HTTP */

